/*
 * Подключаем необходимые библиотеки
 */
#include <Console.h>
#include <Servo.h>
#include <Bridge.h>
#include <Temboo.h>
#include "TembooAccount.h"

#define SERVO_PIN    6
#define DETECTOR_PIN 2
#define BUZZER_PIN   12
#define BUZZER_GND   8

#define SHOOT_DELAY  3000


Servo srv;


void setup()
{
  // Открываем соединение с Linux-частью
  Bridge.begin();
  Console.begin();
  
  // Настраиваем пины пьезо-пищалки
  pinMode(BUZZER_GND, OUTPUT);
  digitalWrite(BUZZER_GND, LOW);
  
  
  // Для отладки пишем приветствие в терминал Linux
  // и даём секундный писк, говорящий о готовности
  Console.println("Hello!");
  tone(BUZZER_PIN, 1000, 1000);
  
  // Настраиваем пин с сервоприводом
  srv.attach(SERVO_PIN);
  
  // Настраиваем пин с сенсором
  pinMode(DETECTOR_PIN, INPUT);
}


/*
 * Функция, которая делает фото веб-камерой и отправляет
 * результат на Facebook
 */
void shoot() 
{
  // Блок фотографирования
  {
    Process p;
   
    // Готовим команду в Linux на фотографирование и
    // сохранение результата в /www/kitty/photo.jpg
    // Не забудьте предварительно создать директорию /www/kitty
    p.begin("/usr/bin/fswebcam");
    p.addParameter("-d");
    p.addParameter("/dev/video0");
    p.addParameter("/www/kitty/photo.jpg");
   
    // Вызываем подготовленную команду и ждём результат
    p.run();
    while(p.available() > 0) {
      char c = p.read();
   
      Console.print(c);
    }
 
    // Результат успешен!
    Console.println("Shooted!");
  }
  
  // Блок загрузки на Facebook
  {
    Process UploadPhotoChoreo;

    // Готовим вызов скрипта Temboo для отправки фото
    UploadPhotoChoreo.begin("sh");
    
    UploadPhotoChoreo.addParameter("/root/fb_upload_photo.sh");
    UploadPhotoChoreo.addParameter(TEMBOO_ACCOUNT);
    UploadPhotoChoreo.addParameter(TEMBOO_APP_KEY_NAME);
    UploadPhotoChoreo.addParameter(TEMBOO_APP_KEY);

    UploadPhotoChoreo.addParameter("asdlfkjasdflkj902342134ioj123kl4ju3890dlksjafiaj23ei2389ej239");
    UploadPhotoChoreo.addParameter("/www/kitty/photo.jpg");

    // Запускаем и ждём результат
    UploadPhotoChoreo.run();
    
    while(UploadPhotoChoreo.available()) {
      char c = UploadPhotoChoreo.read();
      Console.print(c);
    }
    UploadPhotoChoreo.close();
  }

  Console.println("Uploaded!");
}


bool wait_cat(void)
{
  unsigned long tm = millis();
  bool detected = false;
  unsigned long start_detected = millis();
  
  // В течение 60 секунд повторяем одно и то же,
  // в надежде приманить кота
  while (millis() - tm < 60000) {
    int i = 10;
    
    // 10 раз подёргиваем серво
    while (i--) {
      srv.write(90);
      delay(200);

      // проверяем: есть ли кот?
      if (!digitalRead(DETECTOR_PIN)) {
        if (!detected) {
          // если это первое обнаружение кота,
          // не реагируем, но запоминаем, что это
          // произошло (для защиты от ложных срабатываний)
          start_detected = millis();
          detected = true;
        } else {
          if (millis()-start_detected > 500) {
            // кот уже был обнаружен ранее — это точно кот!
            // пищим
            tone(BUZZER_PIN, 500, 500);
            // фотографируем и отправляем
            shoot();
            // паркуем удочку
            srv.write(0);
            
            return true;
          }
        }
      } else if (detected) {
        // нет, это был не кот :(
        detected = false;
      }

      // паркуем удочку
      srv.write(0);
      // ждём пока серво отработает
      delay(200);
    }
    
    detected = false;

    // каждые 3 секунды делаем паузу, чтобы приманка
    // выглядела привлекательнее
    delay(3000);
  }
  
  return false;
}

void loop()
{
  // маним кота
  wait_cat();
  
  // ждём 3 минуты, чтобы не зафлудить Facebook
  delay(3*60000);
}
